import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'prototype_app.dart';
import 'prototype_controller.dart';
import 'prototype_descriptor.dart';
import 'prototype_page_list_scaffold.dart';

class PrototypePresenter extends StatefulWidget {
  const PrototypePresenter({
    Key? key,
    required this.controller,
    this.descriptor,
    required this.showPageInfos,
  }) : super(key: key);

  // final List<MockupPageDescriptor> pages;
  final PrototypeController controller;
  final PrototypeDescriptor? descriptor;
  final bool showPageInfos;

  @override
  _PrototypePresenterState createState() => _PrototypePresenterState();

  static void showPageInfo(PrototypeController controller) {
    PrototypeApp.messenger.currentState!.hideCurrentSnackBar();
    // ScaffoldMessenger.of(context).hideCurrentSnackBar();
//                    ScaffoldMessenger.of(context).showSnackBar(
    PrototypeApp.messenger.currentState!.showSnackBar(
      SnackBar(
        // margin: EdgeInsets.all(0),
        // padding: EdgeInsets.all(0),
        elevation: 4,
        backgroundColor: Colors.grey.shade900,
        behavior: SnackBarBehavior.floating,
        content: Material(
          color: Colors.grey.shade900,
          child: Padding(
            padding: const EdgeInsets.all(0.0),
            child: Card(
              child: ListTile(
                title: Text(controller.currentPage.title),
                contentPadding: EdgeInsets.all(16),
                leading: CircleAvatar(
                  child: Text(controller.currentPage.number.toString()),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class _PrototypePresenterState extends State<PrototypePresenter> {
  late final PrototypeController controller; // = MockupController(pages: []);
  // late bool menuVisible;
  // late bool showPageInfos;

  @override
  void initState() {
    super.initState();
    // menuVisible = false;
    // showPageInfos = true;
    controller = widget.controller;
    // controller.toPage(1);
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<PrototypeController>(
      create: (context) => controller,
      child: Consumer<PrototypeController>(
        builder: (context, value, child) {
          return GestureDetector(
            onLongPress: () {
              controller.showPageMenu();
            },
            child: PageView.builder(
              pageSnapping: true,
              itemCount: controller.descriptor.pages.length,
              controller: controller.pageController,
              itemBuilder: (context, index) {
                if (widget.showPageInfos && controller.currentPageIndex != index && controller.pageMenuVisible == false) {
                  // print('addpostback');
                  WidgetsBinding.instance!.addPostFrameCallback((_) {
                    PrototypePresenter.showPageInfo(controller);
                  });
                }

                controller.setCurrentIndex(index);

                final mq = MediaQuery.of(context);
                return Stack(
                  children: [
                    MediaQuery(data: mq, child: controller.getBuilder(index)(context)),
                    Visibility(
                      visible: controller.pageMenuVisible,
                      maintainAnimation: true,
                      maintainState: true,
                      child: AnimatedOpacity(
                        opacity: controller.pageMenuVisible ? 1.0 : 0.0,
                        duration: const Duration(milliseconds: 500),
                        child: Center(
                          child: PrototypePageListScaffold(
                            controller: controller,
                          ),
                        ),
                      ),
                    ),
                  ],
                );
              },
            ),
          );
        },
      ),
    );
  }
}

class PageInfoCard extends StatefulWidget {
  const PageInfoCard({
    Key? key,
    required this.controller,
  }) : super(key: key);

  final PrototypeController controller;

  @override
  State<PageInfoCard> createState() => _PageInfoCardState();
}

class _PageInfoCardState extends State<PageInfoCard> {
  late double hide;

  @override
  void initState() {
    super.initState();
    hide = 1.0;
  }

  @override
  Widget build(BuildContext context) {
    Future.delayed(Duration(seconds: 1), () {
      if (mounted)
        setState(() {
          hide = 0.0;
        });
      ;
    });

    return Visibility(
      visible: widget.controller.pageInfosVisible,
      maintainAnimation: true,
      maintainState: true,
      child: AnimatedOpacity(
        opacity: hide,
        duration: const Duration(milliseconds: 800),
        child: Container(
          alignment: Alignment.topCenter,
          constraints: BoxConstraints(minWidth: MediaQuery.of(context).size.width),
          // height: 120,
          width: double.infinity,
          child: Material(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Card(
                child: ListTile(
                  title: Text(widget.controller.currentPage.title),
                  contentPadding: EdgeInsets.all(16),
                  leading: CircleAvatar(
                    child: Text(widget.controller.currentPage.number.toString()),
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
